{% extends "layout.html.j2" %}

{% block content %}
<h1 class="fs-5">Login</h1>
<a type="button" class="btn btn-sm btn-discord my-2 w-100" href="{{ discord_oauth }}">
    <i class="bi bi-discord"></i>
    Discord login
</a>
{% endblock %}